@@ -3,17 +3,20 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import argparse
+import json
 import logging
 import os
+import subprocess
 import sys
 from datetime import date, timedelta
-from typing import List, Optional
+from urllib.parse import urlparse
 
 import requests
-from clean_skipfails import CleanSkipfails
 from mach.decorators import Command, CommandArgument, SubCommand
-from mozbuild.base import BuildEnvironmentNotFoundException
+from mozbuild.base import BuildEnvironmentNotFoundException, MozbuildObject
 from mozbuild.base import MachCommandConditions as conditions
+from mozbuild.nodeutil import find_node_executable
+from mozsystemmonitor.resourcemonitor import SystemResourceMonitor
 
 UNKNOWN_TEST = """
 I was unable to find tests from the given argument(s).
@@ -48,7 +51,7 @@ def get_test_parser():
     parser.add_argument(
         "what",
         default=None,
-        nargs="+",
+        nargs="*",
         help=TEST_HELP.format(", ".join(sorted(TEST_SUITES))),
     )
     parser.add_argument(
@@ -66,6 +69,18 @@ def get_test_parser():
         nargs="?",
         help="Specify a debugger to use.",
     )
+    parser.add_argument(
+        "--auto",
+        nargs="?",
+        const="quick",
+        default=False,
+        choices=["extensive", "moderate", "quick"],
+        metavar="LEVEL",
+        help="Automatically select tests based on local changes using BugBug. "
+        "Optional confidence level: 'extensive' (more tests), 'moderate' or "
+        "'quick' (fewer tests with highest confidence to be related). "
+        "Default: quick",
+    )
     add_logging_group(parser)
     return parser
 
@@ -149,7 +164,6 @@ def addtest(
     editor=MISSING_ARG,
     **kwargs,
 ):
-    import io
 
     import addtest
     from moztest.resolve import TEST_SUITES
@@ -183,25 +197,23 @@ def addtest(
     if not suite:
         print(
             "We couldn't automatically determine a suite. "
-            "Please specify `--suite` with one of the following options:\n{}\n"
+            f"Please specify `--suite` with one of the following options:\n{ADD_TEST_SUPPORTED_SUITES}\n"
             "If you'd like to add support to a new suite, please file a bug "
-            "blocking https://bugzilla.mozilla.org/show_bug.cgi?id=1540285.".format(
-                ADD_TEST_SUPPORTED_SUITES
-            )
+            "blocking https://bugzilla.mozilla.org/show_bug.cgi?id=1540285."
         )
         return 1
 
     if doc not in ADD_TEST_SUPPORTED_DOCS:
         print(
             "Error: invalid `doc`. Either pass in a test with a valid extension"
-            "({}) or pass in the `doc` argument".format(ADD_TEST_SUPPORTED_DOCS)
+            f"({ADD_TEST_SUPPORTED_DOCS}) or pass in the `doc` argument"
         )
         return 1
 
     creator_cls = addtest.creator_for_suite(suite)
 
     if creator_cls is None:
-        print("Sorry, `addtest` doesn't currently know how to add {}".format(suite))
+        print(f"Sorry, `addtest` doesn't currently know how to add {suite}")
         return 1
 
     creator = creator_cls(command_context.topsrcdir, test, suite, doc, **kwargs)
@@ -216,14 +228,14 @@ def addtest(
         added_tests = True
         if path:
             paths.append(path)
-            print("Adding a test file at {} (suite `{}`)".format(path, suite))
+            print(f"Adding a test file at {path} (suite `{suite}`)")
 
             try:
                 os.makedirs(os.path.dirname(path))
             except OSError:
                 pass
 
-            with io.open(path, "w", newline="\n") as f:
+            with open(path, "w", newline="\n") as f:
                 f.write(template)
         else:
             # write to stdout if you passed only suite and doc and not a file path
@@ -242,14 +254,12 @@ def addtest(
         mach_command = TEST_SUITES[suite]["mach_command"]
         print(
             "Please make sure to add the new test to your commit. "
-            "You can now run the test with:\n    ./mach {} {}".format(
-                mach_command, test
-            )
+            f"You can now run the test with:\n    ./mach {mach_command} {test}"
         )
 
     if editor is not MISSING_ARG:
         if editor is not None:
-            editor = editor
+            pass
         elif "VISUAL" in os.environ:
             editor = os.environ["VISUAL"]
         elif "EDITOR" in os.environ:
@@ -260,9 +270,7 @@ def addtest(
 
         proc = None
         if editor:
-            import subprocess
-
-            proc = subprocess.Popen("%s %s" % (editor, " ".join(paths)), shell=True)
+            proc = subprocess.Popen(f"{editor} {' '.join(paths)}", shell=True)
 
         if proc:
             proc.wait()
@@ -306,21 +314,18 @@ def guess_suite(abs_test):
         and guess_doc(abs_test) == "js"
     ):
         guessed_suite = "xpcshell"
-    else:
-        if filename.startswith("browser_") and (has_browser_ini or has_browser_toml):
-            guessed_suite = "mochitest-browser-chrome"
-        elif filename.startswith("test_"):
-            if (has_chrome_ini or has_chrome_toml) and (
-                has_plain_ini or has_plain_toml
-            ):
-                err = (
-                    "Error: directory contains both a chrome.{ini|toml} and mochitest.{ini|toml}. "
-                    "Please set --suite=mochitest-chrome or --suite=mochitest-plain."
-                )
-            elif has_chrome_ini or has_chrome_toml:
-                guessed_suite = "mochitest-chrome"
-            elif has_plain_ini or has_plain_toml:
-                guessed_suite = "mochitest-plain"
+    elif filename.startswith("browser_") and (has_browser_ini or has_browser_toml):
+        guessed_suite = "mochitest-browser-chrome"
+    elif filename.startswith("test_"):
+        if (has_chrome_ini or has_chrome_toml) and (has_plain_ini or has_plain_toml):
+            err = (
+                "Error: directory contains both a chrome.toml and mochitest.toml. "
+                "Please set --suite=mochitest-chrome or --suite=mochitest-plain."
+            )
+    elif has_chrome_ini or has_chrome_toml:
+        guessed_suite = "mochitest-chrome"
+    elif has_plain_ini or has_plain_toml:
+        guessed_suite = "mochitest-plain"
     return guessed_suite, err
 
 
@@ -377,9 +382,56 @@ def test(command_context, what, extra_args, **log_args):
     `./mach <test-harness> --help`. For example, `./mach mochitest --help`.
     """
     from mozlog.commandline import setup_logging
-    from mozlog.handlers import StreamHandler
+    from mozlog.handlers import ResourceHandler, StreamHandler
     from moztest.resolve import TEST_SUITES, TestResolver, get_suite_definition
 
+    if not log_args.get("auto") and not what:
+        print("Error: You must specify test paths or use --auto flag")
+        return 1
+
+    if log_args.get("auto"):
+        from itertools import chain
+
+        from gecko_taskgraph.util.bugbug import patch_schedules
+        from mozversioncontrol.factory import get_specific_repository_object
+
+        if what:
+            print(
+                "Note: when using --auto, any test paths specified will be combined with BugBug's recommendations."
+            )
+
+        selection_mode = log_args.get("auto")
+
+        repo = get_specific_repository_object(".", "git")
+        base_commit = repo.base_ref_as_commit()
+        patch = "\n".join(
+            [
+                repo.get_patches_after_ref(base_commit),
+                repo.get_patch_for_uncommitted_changes(),
+            ]
+        )
+        if not patch.strip():
+            print("No local changes detected; no tests to run.")
+            return 1
+
+        print(
+            f"Querying BugBug for test recommendations... (based on changes after {base_commit[:8]})"
+        )
+        schedules = patch_schedules(base_commit, patch, selection_mode)
+        if not schedules:
+            print("BugBug did not recommend any tests for your changes.")
+
+        if not schedules and not what:
+            print("Consider specifying tests by path or suite name.")
+            return 1
+
+        test_paths = sorted(schedules.keys())
+        print(f"BugBug recommended {len(test_paths)} test group(s):")
+        for path in test_paths:
+            print(f"  {path} (confidence: {schedules[path]:.2f})")
+
+        what = set(chain(what, test_paths))
+
     resolver = command_context._spawn(TestResolver)
     run_suites, run_tests = resolver.resolve_metadata(what)
 
@@ -414,6 +466,8 @@ def test(command_context, what, extra_args, **log_args):
         if isinstance(handler, StreamHandler):
             handler.formatter.inner.summary_on_shutdown = True
 
+    log.add_handler(ResourceHandler(command_context))
+
     if log_args.get("custom_handler", None) is not None:
         log.add_handler(log_args.get("custom_handler"))
 
@@ -442,7 +496,7 @@ def test(command_context, what, extra_args, **log_args):
     for (flavor, subsuite), tests in sorted(buckets.items()):
         _, m = get_suite_definition(flavor, subsuite)
         if "mach_command" not in m:
-            substr = "-{}".format(subsuite) if subsuite else ""
+            substr = f"-{subsuite}" if subsuite else ""
             print(UNKNOWN_FLAVOR % (flavor, substr))
             status = 1
             continue
@@ -508,7 +562,9 @@ def run_cppunit_test(command_context, **params):
         )
 
         verify_android_device(command_context, install=InstallIntent.NO)
-        return run_android_test(tests, symbols_path, manifest_path, log)
+        return run_android_test(
+            command_context, tests, symbols_path, manifest_path, log
+        )
 
     return run_desktop_test(
         command_context, tests, symbols_path, manifest_path, utility_path, log
@@ -533,7 +589,7 @@ def run_desktop_test(
     try:
         result = cppunittests.run_test_harness(options, tests)
     except Exception as e:
-        log.error("Caught exception running cpp unit tests: %s" % str(e))
+        log.error(f"Caught exception running cpp unit tests: {str(e)}")
         result = False
         raise
 
@@ -555,9 +611,9 @@ def run_android_test(command_context, tests, symbols_path, manifest_path, log):
     options.symbols_path = symbols_path
     options.manifest_path = manifest_path
     options.xre_path = command_context.bindir
-    options.local_lib = command_context.bindir.replace("bin", "fennec")
+    options.local_lib = command_context.bindir.replace("bin", "geckoview")
     for file in os.listdir(os.path.join(command_context.topobjdir, "dist")):
-        if file.endswith(".apk") and file.startswith("fennec"):
+        if file.endswith(".apk") and file.startswith("geckoview"):
             options.local_apk = os.path.join(command_context.topobjdir, "dist", file)
             log.info("using APK: " + options.local_apk)
             break
@@ -565,7 +621,7 @@ def run_android_test(command_context, tests, symbols_path, manifest_path, log):
     try:
         result = remotecppunittests.run_test_harness(options, tests)
     except Exception as e:
-        log.error("Caught exception running cpp unit tests: %s" % str(e))
+        log.error(f"Caught exception running cpp unit tests: {str(e)}")
         result = False
         raise
 
@@ -589,8 +645,6 @@ def executable_name(name):
     help="Extra arguments to pass down to the test harness.",
 )
 def run_jstests(command_context, shell, params):
-    import subprocess
-
     command_context.virtualenv_manager.ensure()
     python = command_context.virtualenv_manager.python_path
 
@@ -623,8 +677,6 @@ def run_jstests(command_context, shell, params):
     help="Extra arguments to pass down to the test harness.",
 )
 def run_jittests(command_context, shell, cgc, params):
-    import subprocess
-
     command_context.virtualenv_manager.ensure()
     python = command_context.virtualenv_manager.python_path
 
@@ -663,8 +715,6 @@ def run_jittests(command_context, shell, cgc, params):
     "omitted, the entire test suite is executed.",
 )
 def run_jsapitests(command_context, list=False, frontend_only=False, test_name=None):
-    import subprocess
-
     jsapi_tests_cmd = [
         os.path.join(command_context.bindir, executable_name("jsapi-tests"))
     ]
@@ -685,8 +735,6 @@ def run_jsapitests(command_context, list=False, frontend_only=False, test_name=N
 
 
 def run_check_js_msg(command_context):
-    import subprocess
-
     command_context.virtualenv_manager.ensure()
     python = command_context.virtualenv_manager.python_path
 
@@ -725,6 +773,129 @@ def test_info(command_context):
     """
 
 
+class TestInfoNodeRunner(MozbuildObject):
+    """Run TestInfo node tests."""
+
+    def run_node_cmd(self, monitor, days=1, revision=None, output_dir=None):
+        """Run the TestInfo node command."""
+
+        self.test_timings_dir = os.path.join(self.topsrcdir, "testing", "timings")
+        test_runner_script = os.path.join(
+            self.test_timings_dir, "fetch-xpcshell-data.js"
+        )
+
+        # Build the command to run
+        node_binary, _ = find_node_executable()
+        cmd = [node_binary, test_runner_script]
+
+        if revision:
+            cmd.extend(["--revision", revision])
+        else:
+            cmd.extend(["--days", str(days)])
+
+        if output_dir:
+            cmd.extend(["--output-dir", os.path.abspath(output_dir)])
+
+        print(f"Running: {' '.join(cmd)}")
+        print(f"Working directory: {self.test_timings_dir}")
+
+        try:
+            # Run the test runner and capture stdout line by line
+            process = subprocess.Popen(
+                cmd,
+                cwd=self.test_timings_dir,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.STDOUT,
+                text=True,
+                bufsize=1,
+            )
+
+            for line_ in process.stdout:
+                line = line_.rstrip()
+                # Print to console
+                print(line)
+
+                # Add as instant event marker to profile (skip empty lines)
+                if line:
+                    monitor.record_event(line)
+
+            process.wait()
+            return process.returncode
+        except FileNotFoundError:
+            print(
+                "ERROR: Node.js not found. Please ensure Node.js is installed and in your PATH."
+            )
+            return 1
+        except Exception as e:
+            print(f"ERROR: Failed to run TestInfo node command: {e}")
+            return 1
+
+
+@SubCommand(
+    "test-info",
+    "xpcshell-timings",
+    description="Collect timing information for XPCShell test jobs.",
+)
+@CommandArgument(
+    "--days",
+    default=1,
+    help="Number of days to download and aggregate, starting with yesterday",
+)
+@CommandArgument(
+    "--revision",
+    default="",
+    help="revision to fetch data for ('mozilla-central:<revision id>', '<revision id>' for a try push or 'current' to take the revision from the environment)",
+)
+@CommandArgument("--output-dir", help="Path to report file.")
+def test_info_xpcshell_timings(command_context, days, output_dir, revision=None):
+    # Start resource monitoring with 0.1s sampling rate
+    monitor = SystemResourceMonitor(poll_interval=0.1)
+    monitor.start()
+
+    try:
+        # node fetch-xpcshell-data.js --days 1
+        runner = TestInfoNodeRunner.from_environment(
+            cwd=os.getcwd(), detect_virtualenv_mozinfo=False
+        )
+
+        # Handle 'current' special value to use current build's revision
+        if revision == "current":
+            rev = os.environ.get("MOZ_SOURCE_CHANGESET", "")
+            repo = os.environ.get("MOZ_SOURCE_REPO", "")
+
+            if rev and repo:
+                # Extract project name from repository URL
+                # e.g., https://hg.mozilla.org/try -> try
+                # e.g., https://hg.mozilla.org/mozilla-central -> mozilla-central
+                parsed_url = urlparse(repo)
+                project = os.path.basename(parsed_url.path)
+                revision = f"{project}:{rev}"
+        elif revision and ":" not in revision:
+            # Bare revision ID without project prefix - assume it's a try push
+            revision = f"try:{revision}"
+
+        runner.run_node_cmd(
+            monitor, days=days, revision=revision, output_dir=output_dir
+        )
+    finally:
+        # Stop resource monitoring and save profile
+        if output_dir:
+            monitor.stop(upload_dir=output_dir)
+            profile_path = os.path.join(output_dir, "profile_resource-usage.json")
+        else:
+            monitor.stop()
+            # This is where ./mach resource-usage will find the profile.
+            profile_path = command_context._get_state_filename(
+                "profile_build_resources.json"
+            )
+        with open(profile_path, "w", encoding="utf-8", newline="\n") as fh:
+            to_write = json.dumps(monitor.as_profile(), separators=(",", ":"))
+            fh.write(to_write)
+        print(f"Resource usage profile saved to: {profile_path}")
+        if not output_dir:
+            print("View it with: ./mach resource-usage")
+
+
 @SubCommand(
     "test-info",
     "tests",
@@ -943,16 +1114,25 @@ def test_info_testrun_report(command_context, output_file):
         "https://hg.mozilla.org/mozilla-central",
         "https://hg.mozilla.org/try",
     ]:
+        # keep the original format around as data store
         runcounts = ti.get_runcounts()
-        if output_file:
-            output_file = os.path.abspath(output_file)
-            output_dir = os.path.dirname(output_file)
-            if not os.path.isdir(output_dir):
-                os.makedirs(output_dir)
-            with open(output_file, "w") as f:
-                json.dump(runcounts, f)
-        else:
+        if not output_file:
             print(runcounts)
+            return
+
+        output_file = os.path.abspath(output_file)
+        output_dir = os.path.dirname(output_file)
+        if not os.path.isdir(output_dir):
+            os.makedirs(output_dir)
+        with open(output_file, "w") as f:
+            json.dump(runcounts, f)
+
+        # creating custom 1, 7, 30 day artifacts instead
+        for days in [1, 7, 30]:
+            optimized_data = ti.optimize_runcounts_data(runcounts, days)
+            new_output_file = output_file.replace(".json", f"-{days}days.json")
+            with open(new_output_file, "w") as f:
+                json.dump(optimized_data, f)
 
 
 @SubCommand(
@@ -990,17 +1170,14 @@ def test_info_failures(
         return
 
     # get bug info
-    url = (
-        "https://bugzilla.mozilla.org/rest/bug?include_fields=summary,depends_on&id=%s"
-        % bugid
-    )
+    url = f"https://bugzilla.mozilla.org/rest/bug?include_fields=summary,depends_on&id={bugid}"
     r = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
     if r.status_code != 200:
-        print("%s error retrieving url: %s" % (r.status_code, url))
+        print(f"{r.status_code} error retrieving url: {url}")
 
     data = r.json()
     if not data:
-        print("unable to get bugzilla information for %s" % bugid)
+        print(f"unable to get bugzilla information for {bugid}")
         return
 
     summary = data["bugs"][0]["summary"]
@@ -1020,7 +1197,7 @@ def test_info_failures(
     data = []
     for b in buglist:
         url = "https://treeherder.mozilla.org/api/failuresbybug/"
-        url += "?startday=%s&endday=%s&tree=trunk&bug=%s" % (start, end, b)
+        url += f"?startday={start}&endday={end}&tree=trunk&bug={b}"
         r = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
         r.raise_for_status()
 
@@ -1035,13 +1212,12 @@ def test_info_failures(
     # query VCS to get current list of variants:
     import yaml
 
-    url = "https://hg.mozilla.org/mozilla-central/raw-file/tip/taskcluster/kinds/test/variants.yml"
+    url = "https://hg.mozilla.org/mozilla-central/raw-file/default/taskcluster/test_configs/variants.yml"
     r = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
     variants = yaml.safe_load(r.text)
 
     print(
-        "\nQuerying data for bug %s annotated from %s to %s on trunk.\n\n"
-        % (buglist, start, end)
+        f"\nQuerying data for bug {buglist} annotated from {start} to {end} on trunk.\n\n"
     )
     jobs = {}
     lines = {}
@@ -1049,13 +1225,12 @@ def test_info_failures(
         # config = platform/buildtype
         # testsuite (<suite>[-variant][-<chunk>])
         # lines - group by patterns that contain test name
-        config = "%s/%s" % (failure["platform"], failure["build_type"])
-
+        config = f"{failure['platform']}/{failure['build_type']}"
         variant = ""
         suite = ""
         varpos = len(failure["test_suite"])
         for v in variants.keys():
-            var = "-%s" % variants[v]["suffix"]
+            var = f"-{variants[v]['suffix']}"
             if var in failure["test_suite"]:
                 if failure["test_suite"].find(var) < varpos:
                     variant = var
@@ -1071,9 +1246,9 @@ def test_info_failures(
             pass  # if this works, then the last '-X' is a number :)
 
         if suite == "":
-            print("Error: failure to find variant in %s" % failure["test_suite"])
+            print(f"Error: failure to find variant in {failure['test_suite']}")
 
-        job = "%s-%s%s" % (config, suite, variant)
+        job = f"{config}-{suite}{variant}"
         if job not in jobs.keys():
             jobs[job] = 0
         jobs[job] += 1
@@ -1085,11 +1260,11 @@ def test_info_failures(
                 continue
             # strip off timestamp and mozharness status
             parts = line.split("TEST-UNEXPECTED")
-            l = "TEST-UNEXPECTED%s" % parts[-1]
+            l = f"TEST-UNEXPECTED{parts[-1]}"
 
             # only keep 25 characters of the failure, often longer is random numbers
             parts = l.split(testname)
-            l = "%s%s%s" % (parts[0], testname, parts[1][:25])
+            l = parts[0] + testname + parts[1][:25]
 
             hvalue += hash(l)
 
@@ -1104,9 +1279,11 @@ def test_info_failures(
         lines[hvalue]["config"].append(job)
 
     for h in lines.keys():
-        print("%s errors with:" % (len(lines[h]["config"])))
-        for l in lines[h]["lines"]:
-            print(l)
+        print(f"{len(lines[h]['config'])} errors with:")
+        failure_lines = lines[h]["lines"]
+        if len(failure_lines) > 0:
+            for l in failure_lines:
+                print(l)
         else:
             print(
                 "... no failure lines recorded in"
@@ -1116,7 +1293,7 @@ def test_info_failures(
         for job in jobs:
             count = len([x for x in lines[h]["config"] if x == job])
             if count > 0:
-                print("  %s: %s" % (job, count))
+                print(f"  {job}: {count}")
         print("")
 
 
@@ -1184,175 +1361,23 @@ def run_migration_tests(command_context, test_paths=None, **kwargs):
 
 
 @Command(
-    "manifest",
+    "platform-diff",
     category="testing",
-    description="Manifest operations",
-    virtualenv_name="manifest",
+    description="Displays the difference in platforms used for the given task by using the output of the tgdiff artifact",
 )
-def manifest(_command_context):
-    """
-    All functions implemented as subcommands.
-    """
-
-
-@SubCommand(
-    "manifest",
-    "skip-fails",
-    description="Update manifests to skip failing tests",
-)
-@CommandArgument("try_url", nargs=1, help="Treeherder URL for try (please use quotes)")
+@CommandArgument("task_id", help="task_id to fetch the tgdiff from.")
 @CommandArgument(
-    "-b",
-    "--bugzilla",
+    "-r",
+    "--replace",
     default=None,
-    dest="bugzilla",
-    help="Bugzilla instance (or disable)",
+    dest="replace",
+    help='Array of strings to replace from the old platforms to find matches in new platforms. Eg: ["1804=2404", "-qr"] will replace "1804" by "2404" and remove "-qr" before looking at new platforms.',
 )
-@CommandArgument(
-    "-m", "--meta-bug-id", default=None, dest="meta_bug_id", help="Meta Bug id"
-)
-@CommandArgument(
-    "-s",
-    "--turbo",
-    action="store_true",
-    dest="turbo",
-    help="Skip all secondary failures",
-)
-@CommandArgument(
-    "-t", "--save-tasks", default=None, dest="save_tasks", help="Save tasks to file"
-)
-@CommandArgument(
-    "-T", "--use-tasks", default=None, dest="use_tasks", help="Use tasks from file"
-)
-@CommandArgument(
-    "-f",
-    "--save-failures",
-    default=None,
-    dest="save_failures",
-    help="Save failures to file",
-)
-@CommandArgument(
-    "-F",
-    "--use-failures",
-    default=None,
-    dest="use_failures",
-    help="Use failures from file",
-)
-@CommandArgument(
-    "-M",
-    "--max-failures",
-    default=-1,
-    dest="max_failures",
-    help="Maximum number of failures to skip (-1 == no limit)",
-)
-@CommandArgument("-v", "--verbose", action="store_true", help="Verbose mode")
-@CommandArgument(
-    "-d",
-    "--dry-run",
-    action="store_true",
-    help="Determine manifest changes, but do not write them",
-)
-@CommandArgument(
-    "-I",
-    "--implicit-vars",
-    action="store_true",
-    help="Use implicit variables in reftest manifests",
-)
-@CommandArgument(
-    "-n",
-    "--new-version",
-    dest="new_version",
-    help="New version to use for annotations",
-)
-def skipfails(
+def platform_diff(
     command_context,
-    try_url,
-    bugzilla=None,
-    meta_bug_id=None,
-    turbo=False,
-    save_tasks=None,
-    use_tasks=None,
-    save_failures=None,
-    use_failures=None,
-    max_failures=-1,
-    verbose=False,
-    dry_run=False,
-    implicit_vars=False,
-    new_version=None,
+    task_id,
+    replace,
 ):
-    from skipfails import Skipfails
+    from platform_diff import PlatformDiff
 
-    if meta_bug_id is not None:
-        try:
-            meta_bug_id = int(meta_bug_id)
-        except ValueError:
-            meta_bug_id = None
-
-    if max_failures is not None:
-        try:
-            max_failures = int(max_failures)
-        except ValueError:
-            max_failures = -1
-    else:
-        max_failures = -1
-
-    Skipfails(
-        command_context,
-        try_url,
-        verbose,
-        bugzilla,
-        dry_run,
-        turbo,
-        implicit_vars,
-        new_version,
-    ).run(
-        meta_bug_id,
-        save_tasks,
-        use_tasks,
-        save_failures,
-        use_failures,
-        max_failures,
-    )
-
-
-@SubCommand(
-    "manifest",
-    "clean-skip-fails",
-    description="Update manifests to remove skip-if conditions for a specific platform. Only works for TOML manifests.",
-)
-@CommandArgument(
-    "manifest_search_path",
-    nargs=1,
-    help="Path to the folder containing the manifests to update, or the path to a single manifest",
-)
-@CommandArgument(
-    "-o",
-    "--os",
-    default=None,
-    dest="os_name",
-    help="OS to remove (linux, mac, win)",
-)
-@CommandArgument(
-    "-s",
-    "--os_version",
-    default=None,
-    dest="os_version",
-    help="Version of the OS to remove (eg: 18.04 for linux)",
-)
-@CommandArgument(
-    "-p",
-    "--processor",
-    default=None,
-    dest="processor",
-    help="Type of processor architecture to remove (eg: x86)",
-)
-def clean_skipfails(
-    command_context,
-    manifest_search_path: List[str],
-    os_name: Optional[str] = None,
-    os_version: Optional[str] = None,
-    processor: Optional[str] = None,
-):
-    CleanSkipfails(
-        command_context, manifest_search_path[0], os_name, os_version, processor
-    ).run()
+    PlatformDiff(command_context, task_id, replace).run()
