@@ -277,7 +277,7 @@ def run_mochitest_general(
 ):
     from mochitest_options import ALL_FLAVORS
     from mozlog.commandline import setup_logging
-    from mozlog.handlers import StreamHandler
+    from mozlog.handlers import ResourceHandler, StreamHandler
     from moztest.resolve import get_suite_definition
 
     # TODO: This is only strictly necessary while mochitest is using Python
@@ -290,6 +290,10 @@ def run_mochitest_general(
             buildapp = app
             break
 
+    # Force the buildapp to be android if requested
+    if kwargs.get("android"):
+        buildapp = "android"
+
     flavors = None
     if flavor:
         for fname, fobj in ALL_FLAVORS.items():
@@ -329,13 +333,16 @@ def run_mochitest_general(
             format_args["compact"] = False
 
         default_format = command_context._mach_context.settings["test"]["format"]
-        kwargs["log"] = setup_logging(
+        log = setup_logging(
             "mach-mochitest", kwargs, {default_format: sys.stdout}, format_args
         )
-        for handler in kwargs["log"].handlers:
+        kwargs["log"] = log
+        for handler in log.handlers:
             if isinstance(handler, StreamHandler):
                 handler.formatter.inner.summary_on_shutdown = True
 
+        log.add_handler(ResourceHandler(command_context))
+
     driver = command_context._spawn(BuildDriver)
     driver.install_tests()
 
@@ -427,13 +434,13 @@ def run_mochitest_general(
             apps = fobj["enabled_apps"]
             name = fobj["aliases"][0]
             if s:
-                name = "{} --subsuite {}".format(name, s)
+                name = f"{name} --subsuite {s}"
 
             if buildapp not in apps:
                 reason = "requires {}".format(" or ".join(apps))
             else:
                 reason = "excluded by the command line"
-            msg.append("    mochitest -f {} ({})".format(name, reason))
+            msg.append(f"    mochitest -f {name} ({reason})")
         print(SUPPORTED_TESTS_NOT_FOUND.format(buildapp, "\n".join(sorted(msg))))
         return 1
 
