@@ -6,6 +6,7 @@
 
 /**
  * SimpleTest framework object.
+ *
  * @class
  */
 var SimpleTest = {};
@@ -417,7 +418,7 @@ SimpleTest.setExpected();
 
 /**
  * Something like assert.
- **/
+ */
 SimpleTest.ok = function (condition, name) {
   if (arguments.length > 2) {
     const diag = "Too many arguments passed to `ok(condition, name)`";
@@ -501,7 +502,7 @@ SimpleTest.record = function (condition, name, diag, stack, expected) {
 
 /**
  * Roughly equivalent to ok(Object.is(a, b), name)
- **/
+ */
 SimpleTest.is = function (a, b, name) {
   // Be lazy and use Object.is til we want to test a browser without it.
   var pass = Object.is(a, b);
@@ -666,7 +667,7 @@ SimpleTest.info = function (name, message) {
 
 /**
  * Copies of is and isnot with the call to ok replaced by a call to todo.
- **/
+ */
 
 SimpleTest.todo_is = function (a, b, name) {
   var pass = Object.is(a, b);
@@ -686,7 +687,7 @@ SimpleTest.todo_isnot = function (a, b, name) {
 
 /**
  * Makes a test report, returns it as a DIV element.
- **/
+ */
 SimpleTest.report = function () {
   var passed = 0;
   var failed = 0;
@@ -738,7 +739,7 @@ SimpleTest.report = function () {
 
 /**
  * Toggle element visibility
- **/
+ */
 SimpleTest.toggle = function (el) {
   if (computedStyle(el, "display") == "block") {
     el.style.display = "none";
@@ -749,7 +750,7 @@ SimpleTest.toggle = function (el) {
 
 /**
  * Toggle visibility for divs with a specific class.
- **/
+ */
 SimpleTest.toggleByClass = function (cls, evt) {
   var children = document.getElementsByTagName("div");
   var elements = [];
@@ -778,7 +779,7 @@ SimpleTest.toggleByClass = function (cls, evt) {
 
 /**
  * Shows the report in the browser
- **/
+ */
 SimpleTest.showReport = function () {
   var togglePassed = createEl("a", { href: "#" }, "Toggle passed checks");
   var toggleFailed = createEl("a", { href: "#" }, "Toggle failed checks");
@@ -821,7 +822,7 @@ SimpleTest.showReport = function () {
  *
  * When SimpleTest.waitForExplicitFinish is called,
  * explicit SimpleTest.finish() is required.
- **/
+ */
 SimpleTest.waitForExplicitFinish = function () {
   SimpleTest._stopOnLoad = false;
 };
@@ -834,7 +835,7 @@ SimpleTest.waitForExplicitFinish = function () {
  * "SimpleTest.requestLongerTimeout(5)" will give it 5 times as long to
  * finish.
  *
- * @param {Number} factor
+ * @param {number} factor
  *        The multiplication factor to use on the timeout for this test.
  */
 SimpleTest.requestLongerTimeout = function (factor) {
@@ -933,9 +934,8 @@ window.setTimeout = function SimpleTest_setTimeoutShim() {
  * using it.  Such magic timeout values could result in intermittent
  * failures in your test, and are almost never necessary!
  *
- * @param {String} reason
+ * @param {string} reason
  *        A string representation of the reason why the test needs timeouts.
- *
  */
 SimpleTest.requestFlakyTimeout = function (reason) {
   SimpleTest.is(typeof reason, "string", "A valid string reason is expected");
@@ -1081,14 +1081,14 @@ const kTextHtmlPrefixClipboardDataWindows =
 const kTextHtmlSuffixClipboardDataWindows =
   "<!--EndFragment-->\n</body>\n</html>";
 
-/*
+/**
  * Polls the clipboard waiting for the expected value. A known value different than
  * the expected value is put on the clipboard first (and also polled for) so we
  * can be sure the value we get isn't just the expected value because it was already
  * on the clipboard. This only uses the global clipboard and only for text/plain
  * values.
  *
- * @param {String|Function} aExpectedStringOrValidatorFn
+ * @param {string | Function} aExpectedStringOrValidatorFn
  *        The string value that is expected to be on the clipboard, or a
  *        validator function getting expected clipboard data and returning a bool.
  *        If you specify string value, line breakers in clipboard are treated
@@ -1109,9 +1109,9 @@ const kTextHtmlSuffixClipboardDataWindows =
  * @param {Function} aFailureFn
  *        A function called if the expected value isn't found on the clipboard
  *        within 5s. It can also be called if the known value can't be found.
- * @param {String} [aFlavor="text/plain"]
+ * @param {string} [aFlavor="text/plain"]
  *        The flavor to look for.
- * @param {Number} [aTimeout=5000]
+ * @param {number} [aTimeout=5000]
  *        The timeout (in milliseconds) to wait for a clipboard change.
  * @param {boolean} [aExpectFailure=false]
  *        If true, fail if the clipboard contents are modified within the timeout
@@ -1227,7 +1227,14 @@ SimpleTest.promiseClipboardChange = async function (
 
     let errorMsg = `Timed out while polling clipboard for ${
       preExpectedVal ? "initialized" : "requested"
-    } data, got: ${data}`;
+    } data, got: ${
+      flavor == "text/plain"
+        ? data
+            .replaceAll("\\", "\\\\")
+            .replaceAll("\t", "\\t")
+            .replaceAll("\n", "\\n")
+        : data
+    }`;
     SimpleTest.ok(expectFailure, errorMsg);
     if (!expectFailure) {
       throw new Error(errorMsg);
@@ -1271,18 +1278,38 @@ SimpleTest.promiseClipboardChange = async function (
  *        A function returns the result of the condition
  * @param {Function} aCallback
  *        A function called after the condition is passed or timeout.
- * @param {String} aErrorMsg
+ * @param {string} aErrorMsg
  *        The message displayed when the condition failed to pass
  *        before timeout.
+ * @param interval
+ *        The time interval to poll the condition function. Defaults
+ *        to 100ms.
+ * @param maxTries
+ *        The number of times to poll before giving up and rejecting
+ *        if the condition has not yet returned true. Defaults to 30
+ *        (~3 seconds for 100ms intervals)
  */
-SimpleTest.waitForCondition = function (aCond, aCallback, aErrorMsg) {
-  this.promiseWaitForCondition(aCond, aErrorMsg).then(() => aCallback());
+SimpleTest.waitForCondition = function (
+  aCond,
+  aCallback,
+  aErrorMsg,
+  interval = 100,
+  maxTries = 30
+) {
+  this.promiseWaitForCondition(aCond, aErrorMsg, interval, maxTries).then(() =>
+    aCallback()
+  );
 };
-SimpleTest.promiseWaitForCondition = async function (aCond, aErrorMsg) {
-  for (let tries = 0; tries < 30; ++tries) {
+SimpleTest.promiseWaitForCondition = async function (
+  aCond,
+  aErrorMsg,
+  interval = 100,
+  maxTries = 30
+) {
+  for (let tries = 0; tries < maxTries; ++tries) {
     // Wait 100ms between checks.
     await new Promise(resolve => {
-      SimpleTest._originalSetTimeout.apply(window, [resolve, 100]);
+      SimpleTest._originalSetTimeout.apply(window, [resolve, interval]);
     });
 
     let conditionPassed;
@@ -1388,7 +1415,7 @@ SimpleTest.finishWithFailure = function (msg) {
 /**
  * Finishes the tests. This is automatically called, except when
  * SimpleTest.waitForExplicitFinish() has been invoked.
- **/
+ */
 SimpleTest.finish = function () {
   if (SimpleTest._alreadyFinished) {
     var err =
@@ -1703,35 +1730,6 @@ SimpleTest.monitorConsole = function (
   SpecialPowers.registerConsoleListener(listener);
 };
 
-/**
- * Enter this window to legacy unpartitioned storage mode.
- * This disables Storage and cookie partitioning for this frame.
- * This is useful where the test utilized BroadcastChannels
- * to coordinate with opened windows and aren't validating
- * storage partitioning behavior.
- */
-SimpleTest.enableLegacyUnpartitionedStorage = async function () {
-  if (isXOrigin) {
-    await SpecialPowers.pushPrefEnv({
-      set: [
-        [
-          "privacy.partition.always_partition_third_party_non_cookie_storage",
-          false,
-        ],
-      ],
-    });
-    SpecialPowers.wrap(window.document).notifyUserGestureActivation();
-    await SpecialPowers.pushPermissions([
-      {
-        type: "storageAccessAPI",
-        allow: true,
-        context: window.document,
-      },
-    ]);
-    await SpecialPowers.wrap(window.document).requestStorageAccess();
-  }
-};
-
 /**
  * Stop monitoring console output.
  */
@@ -2279,6 +2277,12 @@ function add_setup(generatorFunction) {
   return add_task(generatorFunction, { isSetup: true });
 }
 
+// Import Mochia methods in the test scope
+SpecialPowers.Services.scriptloader.loadSubScript(
+  "resource://testing-common/Mochia.js",
+  this
+);
+
 // Request complete log when using failure patterns so that failure info
 // from infra can be useful.
 if (usesFailurePatterns()) {
