@@ -1,11 +1,11 @@
-import asyncio
-
 import pytest
 
 URL = "https://workhub.transcribeme.com/Exam"
 
+CAPTCHA_CSS = "re-captcha"
 USERNAME_CSS = "input[name=username]"
 PASSWORD_CSS = "input[name=password]"
+EXPIRED_CSS = "app-renewpassword"
 SIGN_IN_CSS = "#btn-login"
 UNSUPPORTED_CSS = "#chromeOnlyPopup"
 
@@ -24,8 +24,24 @@ async def does_unsupported_popup_appear(client, credentials):
     password.send_keys(credentials["password"])
     sign_in.click()
 
-    client.await_css(UNSUPPORTED_CSS)
-    await asyncio.sleep(1)
+    for _ in range(10):
+        captcha, unsupported, expired = client.await_first_element_of(
+            [
+                client.css(CAPTCHA_CSS),
+                client.css(UNSUPPORTED_CSS),
+                client.css(EXPIRED_CSS),
+            ]
+        )
+        if captcha:
+            print("\a")  # beep to let the user know to do the captcha
+            client.await_element_hidden(client.css(CAPTCHA_CSS), timeout=90)
+        else:
+            break
+    if expired:
+        pytest.skip("Your password has expired. Please visit the site and renew it.")
+        return
+
+    await client.stall(1)
     return client.find_css(UNSUPPORTED_CSS, is_displayed=True)
 
 
