@@ -208,8 +208,6 @@ var didRst = false;
 var rstConnection = null;
 var illegalheader_conn = null;
 
-var gDoHPortsLog = [];
-var gDoHNewConnLog = {};
 var gDoHRequestCount = 0;
 
 // eslint-disable-next-line complexity
@@ -219,10 +217,7 @@ function handleRequest(req, res) {
     u = url.parse(req.url, true);
   }
   var content = getHttpContent(u.pathname);
-  var push, push1, push1a, push2, push3;
-
-  // PushService tests.
-  var pushPushServer1, pushPushServer2, pushPushServer3, pushPushServer4;
+  var push;
 
   function createCNameContent(payload) {
     let packet = dnsPacket.decode(payload);
@@ -523,7 +518,7 @@ function handleRequest(req, res) {
         arg => {
           writeResponse(arg[0], arg[1]);
         },
-        delay,
+        delay + 1,
         [response, buf]
       );
       return;
@@ -615,104 +610,6 @@ function handleRequest(req, res) {
     return;
   } else if (u.pathname === "/cookie_crumbling") {
     res.setHeader("X-Received-Header-Pairs", JSON.stringify(decompressedPairs));
-  } else if (u.pathname === "/push") {
-    push = res.push("/push.js");
-    push.writeHead(200, {
-      "content-type": "application/javascript",
-      pushed: "yes",
-      "content-length": 11,
-      "X-Connection-Http2": "yes",
-    });
-    push.end("// comments");
-    content = '<head> <script src="push.js"/></head>body text';
-  } else if (u.pathname === "/push.js") {
-    content = "// comments";
-    res.setHeader("pushed", "no");
-  } else if (u.pathname === "/push2") {
-    push = res.push("/push2.js");
-    push.writeHead(200, {
-      "content-type": "application/javascript",
-      pushed: "yes",
-      // no content-length
-      "X-Connection-Http2": "yes",
-    });
-    push.end("// comments");
-    content = '<head> <script src="push2.js"/></head>body text';
-  } else if (u.pathname === "/push5") {
-    push = res.push("/push5.js");
-    push.writeHead(200, {
-      "content-type": "application/javascript",
-      pushed: "yes",
-      // no content-length
-      "X-Connection-Http2": "yes",
-    });
-    content = generateContent(1024 * 150);
-    push.write(content);
-    push.end();
-    content = '<head> <script src="push5.js"/></head>body text';
-  } else if (u.pathname === "/pushapi1") {
-    push1 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushapi1/1",
-      method: "GET",
-      headers: { "x-pushed-request": "true", "x-foo": "bar" },
-    });
-    push1.writeHead(200, {
-      pushed: "yes",
-      "content-length": 1,
-      subresource: "1",
-      "X-Connection-Http2": "yes",
-    });
-    push1.end("1");
-
-    push1a = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushapi1/1",
-      method: "GET",
-      headers: { "x-foo": "bar", "x-pushed-request": "true" },
-    });
-    push1a.writeHead(200, {
-      pushed: "yes",
-      "content-length": 1,
-      subresource: "1a",
-      "X-Connection-Http2": "yes",
-    });
-    push1a.end("1");
-
-    push2 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushapi1/2",
-      method: "GET",
-      headers: { "x-pushed-request": "true" },
-    });
-    push2.writeHead(200, {
-      pushed: "yes",
-      subresource: "2",
-      "content-length": 1,
-      "X-Connection-Http2": "yes",
-    });
-    push2.end("2");
-
-    push3 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushapi1/3",
-      method: "GET",
-      headers: { "x-pushed-request": "true", "Accept-Encoding": "br" },
-    });
-    push3.writeHead(200, {
-      pushed: "yes",
-      "content-length": 6,
-      subresource: "3",
-      "content-encoding": "br",
-      "X-Connection-Http2": "yes",
-    });
-    push3.end(Buffer.from([0x8b, 0x00, 0x80, 0x33, 0x0a, 0x03])); // '3\n'
-
-    content = "0";
   } else if (u.pathname === "/big") {
     content = generateContent(128 * 1024);
     var hash = crypto.createHash("md5");
@@ -940,13 +837,6 @@ function handleRequest(req, res) {
       emitResponse(res, payload);
     });
     return;
-  } else if (u.pathname == "/get-doh-req-port-log") {
-    let rContent = JSON.stringify(gDoHPortsLog);
-    res.setHeader("Content-Type", "text/plain");
-    res.setHeader("Content-Length", rContent.length);
-    res.writeHead(400);
-    res.end(rContent);
-    return;
   } else if (u.pathname == "/reset-doh-request-count") {
     gDoHRequestCount = 0;
     res.setHeader("Content-Type", "text/plain");
@@ -1065,29 +955,7 @@ function handleRequest(req, res) {
       // parload is empty when we send redirect response.
       if (payload.length) {
         let packet = dnsPacket.decode(payload);
-        let delay;
-        if (u.query.conncycle) {
-          let name = packet.questions[0].name;
-          if (name.startsWith("newconn")) {
-            // If we haven't seen a req for this newconn name before,
-            // or if we've seen one for the same name on the same port,
-            // synthesize a timeout.
-            if (
-              !gDoHNewConnLog[name] ||
-              gDoHNewConnLog[name] == req.remotePort
-            ) {
-              delay = 1000;
-            }
-            if (!gDoHNewConnLog[name]) {
-              gDoHNewConnLog[name] = req.remotePort;
-            }
-          }
-          gDoHPortsLog.push([packet.questions[0].name, req.remotePort]);
-        } else {
-          gDoHPortsLog = [];
-          gDoHNewConnLog = {};
-        }
-        emitResponse(res, payload, packet, delay);
+        emitResponse(res, payload, packet);
       }
     });
     return;
@@ -1280,43 +1148,6 @@ function handleRequest(req, res) {
     res.writeHead(101);
     res.end("something....");
     return;
-  }
-  // for use with test_dns_by_type_resolve.js
-  else if (u.pathname === "/txt-dns-push") {
-    // _esni_push.example.com has A entry 127.0.0.1
-    let rContent = Buffer.from(
-      "0000010000010001000000000A5F65736E695F70757368076578616D706C6503636F6D0000010001C00C000100010000003700047F000001",
-      "hex"
-    );
-
-    // _esni_push.example.com has TXT entry 2062586B67646D39705932556761584D6762586B676347467A63336476636D513D
-    var pcontent = Buffer.from(
-      "0000818000010001000000000A5F65736E695F70757368076578616D706C6503636F6D0000100001C00C001000010000003700212062586B67646D39705932556761584D6762586B676347467A63336476636D513D",
-      "hex"
-    );
-
-    push = res.push({
-      hostname: "foo.example.com:" + serverPort,
-      port: serverPort,
-      path: "/dns-pushed-response?dns=AAABAAABAAAAAAABCl9lc25pX3B1c2gHZXhhbXBsZQNjb20AABAAAQAAKRAAAAAAAAAIAAgABAABAAA",
-      method: "GET",
-      headers: {
-        accept: "application/dns-message",
-      },
-    });
-    push.writeHead(200, {
-      "content-type": "application/dns-message",
-      pushed: "yes",
-      "content-length": pcontent.length,
-      "X-Connection-Http2": "yes",
-    });
-    push.end(pcontent);
-    res.setHeader("Content-Type", "application/dns-message");
-    res.setHeader("Content-Length", rContent.length);
-    res.writeHead(200);
-    res.write(rContent);
-    res.end("");
-    return;
   } else if (u.pathname === "/.well-known/http-opportunistic") {
     res.setHeader("Cache-Control", "no-cache");
     res.setHeader("Content-Type", "application/json");
@@ -1334,189 +1165,6 @@ function handleRequest(req, res) {
     res.writeHead(200, "OK");
     res.end("1");
     return;
-  }
-
-  // for PushService tests.
-  else if (u.pathname === "/pushSubscriptionSuccess/subscribe") {
-    res.setHeader(
-      "Location",
-      "https://localhost:" + serverPort + "/pushSubscriptionSuccesss"
-    );
-    res.setHeader(
-      "Link",
-      '</pushEndpointSuccess>; rel="urn:ietf:params:push", ' +
-        '</receiptPushEndpointSuccess>; rel="urn:ietf:params:push:receipt"'
-    );
-    res.writeHead(201, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/pushSubscriptionSuccesss") {
-    // do nothing.
-    return;
-  } else if (u.pathname === "/pushSubscriptionMissingLocation/subscribe") {
-    res.setHeader(
-      "Link",
-      '</pushEndpointMissingLocation>; rel="urn:ietf:params:push", ' +
-        '</receiptPushEndpointMissingLocation>; rel="urn:ietf:params:push:receipt"'
-    );
-    res.writeHead(201, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/pushSubscriptionMissingLink/subscribe") {
-    res.setHeader(
-      "Location",
-      "https://localhost:" + serverPort + "/subscriptionMissingLink"
-    );
-    res.writeHead(201, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/pushSubscriptionLocationBogus/subscribe") {
-    res.setHeader("Location", "1234");
-    res.setHeader(
-      "Link",
-      '</pushEndpointLocationBogus; rel="urn:ietf:params:push", ' +
-        '</receiptPushEndpointLocationBogus>; rel="urn:ietf:params:push:receipt"'
-    );
-    res.writeHead(201, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/pushSubscriptionMissingLink1/subscribe") {
-    res.setHeader(
-      "Location",
-      "https://localhost:" + serverPort + "/subscriptionMissingLink1"
-    );
-    res.setHeader(
-      "Link",
-      '</receiptPushEndpointMissingLink1>; rel="urn:ietf:params:push:receipt"'
-    );
-    res.writeHead(201, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/pushSubscriptionMissingLink2/subscribe") {
-    res.setHeader(
-      "Location",
-      "https://localhost:" + serverPort + "/subscriptionMissingLink2"
-    );
-    res.setHeader(
-      "Link",
-      '</pushEndpointMissingLink2>; rel="urn:ietf:params:push"'
-    );
-    res.writeHead(201, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/subscriptionMissingLink2") {
-    // do nothing.
-    return;
-  } else if (u.pathname === "/pushSubscriptionNot201Code/subscribe") {
-    res.setHeader(
-      "Location",
-      "https://localhost:" + serverPort + "/subscriptionNot2xxCode"
-    );
-    res.setHeader(
-      "Link",
-      '</pushEndpointNot201Code>; rel="urn:ietf:params:push", ' +
-        '</receiptPushEndpointNot201Code>; rel="urn:ietf:params:push:receipt"'
-    );
-    res.writeHead(200, "OK");
-    res.end("");
-    return;
-  } else if (u.pathname === "/pushNotifications/subscription1") {
-    pushPushServer1 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushNotificationsDeliver1",
-      method: "GET",
-      headers: {
-        "Encryption-Key":
-          'keyid="notification1"; dh="BO_tgGm-yvYAGLeRe16AvhzaUcpYRiqgsGOlXpt0DRWDRGGdzVLGlEVJMygqAUECarLnxCiAOHTP_znkedrlWoU"',
-        Encryption: 'keyid="notification1";salt="uAZaiXpOSfOLJxtOCZ09dA"',
-        "Content-Encoding": "aesgcm128",
-      },
-    });
-    pushPushServer1.writeHead(200, {
-      subresource: "1",
-    });
-
-    pushPushServer1.end(
-      "370aeb3963f12c4f12bf946bd0a7a9ee7d3eaff8f7aec62b530fc25cfa",
-      "hex"
-    );
-    return;
-  } else if (u.pathname === "/pushNotifications/subscription2") {
-    pushPushServer2 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushNotificationsDeliver3",
-      method: "GET",
-      headers: {
-        "Encryption-Key":
-          'keyid="notification2"; dh="BKVdQcgfncpNyNWsGrbecX0zq3eHIlHu5XbCGmVcxPnRSbhjrA6GyBIeGdqsUL69j5Z2CvbZd-9z1UBH0akUnGQ"',
-        Encryption: 'keyid="notification2";salt="vFn3t3M_k42zHBdpch3VRw"',
-        "Content-Encoding": "aesgcm128",
-      },
-    });
-    pushPushServer2.writeHead(200, {
-      subresource: "1",
-    });
-
-    pushPushServer2.end(
-      "66df5d11daa01e5c802ff97cdf7f39684b5bf7c6418a5cf9b609c6826c04b25e403823607ac514278a7da945",
-      "hex"
-    );
-    return;
-  } else if (u.pathname === "/pushNotifications/subscription3") {
-    pushPushServer3 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushNotificationsDeliver3",
-      method: "GET",
-      headers: {
-        "Encryption-Key":
-          'keyid="notification3";dh="BD3xV_ACT8r6hdIYES3BJj1qhz9wyv7MBrG9vM2UCnjPzwE_YFVpkD-SGqE-BR2--0M-Yf31wctwNsO1qjBUeMg"',
-        Encryption:
-          'keyid="notification3"; salt="DFq188piWU7osPBgqn4Nlg"; rs=24',
-        "Content-Encoding": "aesgcm128",
-      },
-    });
-    pushPushServer3.writeHead(200, {
-      subresource: "1",
-    });
-
-    pushPushServer3.end(
-      "2caaeedd9cf1059b80c58b6c6827da8ff7de864ac8bea6d5775892c27c005209cbf9c4de0c3fbcddb9711d74eaeebd33f7275374cb42dd48c07168bc2cc9df63e045ce2d2a2408c66088a40c",
-      "hex"
-    );
-    return;
-  } else if (u.pathname == "/pushNotifications/subscription4") {
-    pushPushServer4 = res.push({
-      hostname: "localhost:" + serverPort,
-      port: serverPort,
-      path: "/pushNotificationsDeliver4",
-      method: "GET",
-      headers: {
-        "Crypto-Key":
-          'keyid="notification4";dh="BJScXUUTcs7D8jJWI1AOxSgAKkF7e56ay4Lek52TqDlWo1yGd5czaxFWfsuP4j7XNWgGYm60-LKpSUMlptxPFVQ"',
-        Encryption: 'keyid="notification4"; salt="sn9p2QqF3V6KBclda8vx7w"',
-        "Content-Encoding": "aesgcm",
-      },
-    });
-    pushPushServer4.writeHead(200, {
-      subresource: "1",
-    });
-
-    pushPushServer4.end(
-      "9eba7ba6192544a39bd9e9b58e702d0748f1776b27f6616cdc55d29ed5a015a6db8f2dd82cd5751a14315546194ff1c18458ab91eb36c9760ccb042670001fd9964557a079553c3591ee131ceb259389cfffab3ab873f873caa6a72e87d262b8684c3260e5940b992234deebf57a9ff3a8775742f3cbcb152d249725a28326717e19cce8506813a155eff5df9bdba9e3ae8801d3cc2b7e7f2f1b6896e63d1fdda6f85df704b1a34db7b2dd63eba11ede154300a318c6f83c41a3d32356a196e36bc905b99195fd91ae4ff3f545c42d17f1fdc1d5bd2bf7516d0765e3a859fffac84f46160b79cedda589f74c25357cf6988cd8ba83867ebd86e4579c9d3b00a712c77fcea3b663007076e21f9819423faa830c2176ff1001c1690f34be26229a191a938517",
-      "hex"
-    );
-    return;
-  } else if (
-    u.pathname === "/pushNotificationsDeliver1" ||
-    u.pathname === "/pushNotificationsDeliver2" ||
-    u.pathname === "/pushNotificationsDeliver3"
-  ) {
-    res.writeHead(410, "GONE");
-    res.end("");
-    return;
   } else if (u.pathname === "/illegalhpacksoft") {
     // This will cause the compressor to compress a header that is not legal,
     // but only affects the stream, not the session.
@@ -1585,113 +1233,16 @@ function handleRequest(req, res) {
     if (req.headers["if-none-match"]) {
       res.setHeader("x-conditional", "true");
     }
-  } else if (u.pathname === "/origin-4") {
-    let originList = [];
-    req.stream.connection.originFrame(originList);
-    res.setHeader("x-client-port", req.remotePort);
-  } else if (u.pathname === "/origin-6") {
-    let originList = [
-      "https://alt1.example.com:" + serverPort,
-      "https://alt2.example.com:" + serverPort,
-      "https://bar.example.com:" + serverPort,
-    ];
-    req.stream.connection.originFrame(originList);
-    res.setHeader("x-client-port", req.remotePort);
-  } else if (u.pathname === "/origin-11-a") {
-    res.setHeader("x-client-port", req.remotePort);
-
-    const pushb = res.push({
-      hostname: "foo.example.com:" + serverPort,
-      port: serverPort,
-      path: "/origin-11-b",
-      method: "GET",
-      headers: { "x-pushed-request": "true", "x-foo": "bar" },
-    });
-    pushb.writeHead(200, {
-      pushed: "yes",
-      "content-length": 1,
-    });
-    pushb.end("1");
-
-    const pushc = res.push({
-      hostname: "bar.example.com:" + serverPort,
-      port: serverPort,
-      path: "/origin-11-c",
-      method: "GET",
-      headers: { "x-pushed-request": "true", "x-foo": "bar" },
-    });
-    pushc.writeHead(200, {
-      pushed: "yes",
-      "content-length": 1,
-    });
-    pushc.end("1");
-
-    const pushd = res.push({
-      hostname: "madeup.example.com:" + serverPort,
-      port: serverPort,
-      path: "/origin-11-d",
-      method: "GET",
-      headers: { "x-pushed-request": "true", "x-foo": "bar" },
-    });
-    pushd.writeHead(200, {
-      pushed: "yes",
-      "content-length": 1,
-    });
-    pushd.end("1");
-
-    const pushe = res.push({
-      hostname: "alt1.example.com:" + serverPort,
-      port: serverPort,
-      path: "/origin-11-e",
-      method: "GET",
-      headers: { "x-pushed-request": "true", "x-foo": "bar" },
-    });
-    pushe.writeHead(200, {
-      pushed: "yes",
-      "content-length": 1,
-    });
-    pushe.end("1");
-  } else if (u.pathname.substring(0, 8) === "/origin-") {
-    // test_origin.js coalescing
-    res.setHeader("x-client-port", req.remotePort);
   } else if (u.pathname === "/statusphrase") {
     // Fortunately, the node-http2 API is dumb enough to allow this right on
     // through, so we can easily test rejecting this on gecko's end.
     res.writeHead("200 OK");
     res.end(content);
     return;
-  } else if (u.pathname === "/doublepush") {
-    push1 = res.push("/doublypushed");
-    push1.writeHead(200, {
-      "content-type": "text/plain",
-      pushed: "yes",
-      "content-length": 6,
-      "X-Connection-Http2": "yes",
-    });
-    push1.end("pushed");
-
-    push2 = res.push("/doublypushed");
-    push2.writeHead(200, {
-      "content-type": "text/plain",
-      pushed: "yes",
-      "content-length": 6,
-      "X-Connection-Http2": "yes",
-    });
-    push2.end("pushed");
   } else if (u.pathname === "/doublypushed") {
     content = "not pushed";
   } else if (u.pathname === "/diskcache") {
     content = "this was pulled via h2";
-  } else if (u.pathname === "/pushindisk") {
-    var pushedContent = "this was pushed via h2";
-    push = res.push("/diskcache");
-    push.writeHead(200, {
-      "content-type": "text/html",
-      pushed: "yes",
-      "content-length": pushedContent.length,
-      "X-Connection-Http2": "yes",
-    });
-    push.end(pushedContent);
   }
 
   // For test_header_Server_Timing.js
@@ -1904,10 +1455,10 @@ let httpServer = http.createServer((req, res) => {
         return;
       }
 
+      let messageId = makeid(6);
       new Promise((resolve, reject) => {
-        forked.resolve = resolve;
-        forked.reject = reject;
-        forked.send({ code });
+        forked.messageHandlers[messageId] = { resolve, reject };
+        forked.send({ code, messageId });
       })
         .then(x => sendBackResponse(x))
         .catch(e => computeAndSendBackResponse(undefined, e));
@@ -1956,11 +1507,13 @@ function forkProcess() {
 function forkProcessInternal(forked) {
   let id = makeid(6);
   forked.errors = "";
+  forked.messageHandlers = {};
   globalObjects[id] = forked;
   forked.on("message", msg => {
-    if (forked.resolve) {
-      forked.resolve(msg);
-      forked.resolve = null;
+    if (msg.messageId && forked.messageHandlers[msg.messageId]) {
+      let handler = forked.messageHandlers[msg.messageId];
+      delete forked.messageHandlers[msg.messageId];
+      handler.resolve(msg);
     } else {
       console.log(
         `forked process without handler sent: ${JSON.stringify(msg)}`
@@ -1979,22 +1532,27 @@ function forkProcessInternal(forked) {
       return;
     }
 
-    if (!forked.reject) {
-      console.log(
-        `child process ${id} closing code: ${code} signal: ${signal}`
-      );
-      return;
-    }
-
+    let errorMsg = `child process exit closing code: ${code} signal: ${signal}`;
     if (forked.errors != "") {
-      forked.reject(forked.errors);
+      errorMsg = forked.errors;
       forked.errors = "";
+    }
+
+    // Handle /kill/ case where forked.reject is set
+    if (forked.reject) {
+      forked.reject(errorMsg);
       forked.reject = null;
+      forked.resolve = null;
+    }
+
+    if (Object.keys(forked.messageHandlers).length === 0) {
       return;
     }
 
-    forked.reject(`child process exit closing code: ${code} signal: ${signal}`);
-    forked.reject = null;
+    for (let messageId in forked.messageHandlers) {
+      forked.messageHandlers[messageId].reject(errorMsg);
+    }
+    forked.messageHandlers = {};
   };
 
   forked.on("error", exitFunction);
