@@ -12,11 +12,11 @@
 import os
 import pprint
 import re
-import shutil
 import sys
 import tempfile
 import unittest
 
+import mozfile
 import mozinfo
 import six
 from mozlog import structured
@@ -485,7 +485,7 @@ class XPCShellTestsTests(unittest.TestCase):
         self.x.harness_timeout = 30 if not mozinfo.info["ccov"] else 60
 
     def tearDown(self):
-        shutil.rmtree(self.tempdir)
+        mozfile.remove(self.tempdir)
         self.x.shutdownNode()
 
     def writeFile(self, name, contents, mode="w"):
@@ -500,7 +500,7 @@ class XPCShellTestsTests(unittest.TestCase):
 
     def writeManifest(self, tests, prefs=[]):
         """
-        Write an xpcshell.ini in the temp directory and set
+        Write an xpcshell.toml in the temp directory and set
         self.manifest to its pathname. |tests| is a list containing
         either strings (for test names), or tuples with a test name
         as the first element and manifest conditions as the following
@@ -509,9 +509,7 @@ class XPCShellTestsTests(unittest.TestCase):
         """
         testlines = []
         for t in tests:
-            testlines.append(
-                '["%s"]' % (t if isinstance(t, six.string_types) else t[0])
-            )
+            testlines.append('["%s"]' % (t if isinstance(t, str) else t[0]))
             if isinstance(t, tuple):
                 testlines.extend(t[1:])
         prefslines = []
@@ -544,20 +542,29 @@ prefs = [
         kwargs["shuffle"] = shuffle
         kwargs["verbose"] = verbose
         kwargs["headless"] = headless
-        kwargs["sequential"] = True
+        kwargs["selfTest"] = True  # Prevent singleFile from forcing sequential=True
         kwargs["testingModulesDir"] = self.testing_modules
         kwargs["utility_path"] = self.utility_path
         kwargs["repeat"] = 0
-        self.assertEqual(
-            expected,
-            self.x.runTests(kwargs),
-            msg="""Tests should have %s, log:
+        # Don't retry tests that are expected to fail
+        if not expected:
+            kwargs["retry"] = False
+
+        startup_profiling = os.environ.pop("MOZ_PROFILER_STARTUP", None)
+        try:
+            self.assertEqual(
+                expected,
+                self.x.runTests(kwargs),
+                msg="""Tests should have %s, log:
 ========
 %s
 ========
 """
-            % ("passed" if expected else "failed", self.log.getvalue()),
-        )
+                % ("passed" if expected else "failed", self.log.getvalue()),
+            )
+        finally:
+            if startup_profiling:
+                os.environ["MOZ_PROFILER_STARTUP"] = startup_profiling
 
     def _assertLog(self, s, expected):
         l = self.log.getvalue()
